/**
 * @author alexander.farkas + philipp.paul
 */
(function($){
	$.widget('ui.scroller', {
		options: {
			//Wrapper Classes:
			hidingWrapper: 'div.rack',
			moveWrapper: 'div.rack-design',
			//Elements Classes
			atoms: 'div.rack-teaser',
			nextLink: 'div.next',
			prevLink: 'div.prev',
			
			activeLinkClass: 'show',
			stickyFirstLast: false,
			
			linkFn: $.noop,
			moveStep: 'atom',
			swipeStep: false,
			direction: 'horizontal',
			
			hidingWidth: false,
			hidingHeight: false,
			dynamicDim: false,
			//animate
			animate: true,
			animateOptions: {
				duration: 600,
				complete: $.noop
			},
			enableMwheel: true,
			
			diashow: false,
			restartDiaShow: true,
			
			addSubPixel: 1,
			addSubPixelPerTeaser: 0.3,
			
			recalcStageOnResize: true,
			recalcTeaserOnResize: false,
			stageTeaser: false,
			updateOnImgLoad: true,
			bindStyle: 'bind',
			
			pagination: false,
			paginationType: 'teaser',
			paginationAtoms: '<li class="pa-$number"><button title="$title"><span>$number</span></button></li>',
			paginationTitleFrom: false,
			activePaginationClass: 'on',
			paginationFn: false,
			defaultSelected: false
		},
		customEvents: ['init', 'create', 'startleft', 'start', 'slide', 'end', 'endreached', 'startreached', 'endleft', 'diashowpaused', 'diashowstopped'],
		_create: function(){
			this._getMarkupOptions();
			var elem = this.element[0],
				o = this.options,
				that = this,
				fn		= o.animateOptions.complete
			;
			
			o.animateOptions.complete = function(){
				if(fn && $.isFunction(fn)){
					fn.call(this, that);
				}
				that.propagate('end');
			};
			
			o.direction = (o.direction == 'vertical') ? {
				scroll: 'scrollTop',
				outerD: 'outerHeight',
				dim: 'height',
				dir: 'Top'
			} : {
				scroll: 'scrollLeft',
				outerD: 'outerWidth',
				dim: 'width',
				dir: 'Left'
			};
			
			this.moveElem = $(o.moveWrapper, elem);
			this.atomElem = $(o.atoms, elem);
			this.hidingWrapper = $(o.hidingWrapper, elem);
			
			this.nextLink = $(o.nextLink, elem);
			this.prevLink = $(o.prevLink, elem);
			
			this.position = 0;
			this.atomPos = 0;
			this.percentage = 0;
			this.oldPosition = 0;
			this.oldAtomPos = 0;
			
			$(this.atomElem[0]).addClass('first-teaser');
			$(this.atomElem[this.atomElem.length - 1]).addClass('last-teaser');
			
			if (o.hidingHeight || o.hidingWidth) {
				var css = (o.hidingHeight) ? {
					height: o.hidingHeight
				} : {};
				if ((o.hidingWidth)) {
					css = $.extend(css, {
						width: o.hidingWidth
					});
				}
				this.hidingWrapper.css(css);
			}
			
			this.selectedFocus = false;
			
			
			if($.fn.setFocus && $.fn.closest){
				var traverse = {};
				if((o.direction.dir === 'Top')){
					traverse[$.ui.keyCode.UP] = 'prev';
					traverse[$.ui.keyCode.DOWN] = 'next';
				} else {
					traverse[$.ui.keyCode.LEFT] = 'prev';
					traverse[$.ui.keyCode.RIGHT] = 'next';
				}
				this.moveElem
					.bind('keyfocus', function(e){
						var atom = $(e.target).closest(o.atoms);
						if(atom[0]){
							that.scrollIntoView(atom);
						}
					})
					.bind('focusin', function(e){
						var atom = $(e.target).closest(o.atoms);
						that.selectedFocus = (atom[0]) ? atom : false;
					})
					.bind('focusout', function(e){
						that.selectedFocus = false;
					})
					.bind('keydown', function(e){
						
						if(that.selectedFocus === false || !traverse[e.keyCode]){return;}
						var selectElement = that.selectedFocus[traverse[e.keyCode]](o.atoms);
						
						if(selectElement && selectElement[0]){
							e.preventDefault();
							selectElement.setFocus();
							that.scrollIntoView(selectElement);
						} else if(that.isSliding){
							e.preventDefault();
						}

					});
			}
			
			this.dims = [0];
			this.hidingWrapper[o.direction.scroll](0);
			this.minPos = 0;
			this.update();
			if(o.stageTeaser){
				o.recalcTeaserOnResize = true;
			}		
			if(o.recalcStageOnResize || o.recalcTeaserOnResize){
				var firstEmChange = true;
				var update = $.Aperto.throttle(function(e){
					if(firstEmChange && e.type == 'emchange'){
						firstEmChange = false;
						return;	
					}
					if(o.recalcTeaserOnResize){
						that.update.call(that, true);
					} else {
						that.stageWidthUpdate.call(that);
					}
					that.updateViewPagination();
				}, 20);
				$(window).bind('resize', update);
				$(document).bind('orientationchange emchange', update);
			}
			
			if(o.updateOnImgLoad){
				this.updateOnImgLoad();
			}
			
			if (o.diashow) {
				this.startDiashow();
				var diashowPause = function(){
					that.pauseDiashow();
				};
				var isPlaying = function(){
					return (!$.prop(this, 'pause') && !$.prop(this, 'ended'));
				};
				var diashowRestart = function(){
					if(o.restartDiaShow && !that.isDiashowStopped && that.isDiashowPaused && !$('video, audio', that.element).filter(isPlaying)[0]) {
						that.startDiashow.call(that);
					}
				};
				
				this.element
					.enterLeave(diashowPause, diashowRestart)
					.bind('play playing', diashowPause)
				;
				
			} else {
				this.stopDiashow();
			}
			
			if (o.enableMwheel && $.fn.mwheelIntent) {
				this.hidingWrapper.mwheelIntent(function(e, d){
					that.stopDiashow.call(that);
					d = (d < 0) ? '-' : '+';
					if((that.position >= that.maxPos && d === '-') || (d === '+' && that.position <= that.minPos)){
						return !that.isSliding;
					}
					
					var moveStep = (o.moveStep) ? o.moveStep : 'atom';
					that.moveTo(d + 'atom1');
					return false;
				});
			}
			o.swipeStep = o.swipeStep || o.moveStep;
			
			if($.fn.swipe){
				var swipe = {
					prev: function(){
						that.pauseDiashow();
						that.moveTo('-'+o.swipeStep);
					},
					next: function(){
						that.pauseDiashow();
						that.moveTo('+'+o.swipeStep);
					}
				};
				this.hidingWrapper.swipe($.extend( {triggerOnTouchEnd: false},
					(o.direction.dir == 'Left') ? 
						{
							swipeLeft: swipe.prev,
							swipeRight: swipe.next
						} :
						{
							swipeUp: swipe.prev,
							swipeDown: swipe.next
						}
				, o.swipeOptions || {}));
			}
			
			$(window).bind('hashchange', function(){
				that.gotoHash();
			});
						
			var handlePrevNext = function(){
				var dir = ($.inArray(this, that.prevLink) !== -1) ?
					'+' :
					'-';
				that.pauseDiashow.call(that);
				that.moveTo(dir + o.moveStep);
				return false;
			};
			
			this.nextLink
				.bind('click.uiscroller', handlePrevNext);
			this.prevLink
				.bind('click.uiscroller', handlePrevNext);
			if($.browser.msie && parseInt($.browser.version, 10) < 7){
				var over = function(){$(this).addClass('over');},
					out = function(){$(this).removeClass('over');}
				;
				this.nextLink
					.hover(over, out);
				this.prevLink
					.hover(over, out);
			}
			
			if(!this.gotoHash(true) && (o.defaultSelected || o.defaultSelected === 0)){
				this.moveTo('goTo'+ o.defaultSelected, false, false, true);
			}
			if($.fn.lazyImgLoader){
				this.element.lazyImgLoader({e: 'scrollerstart', visible: this.atomElem.eq(this.atomPos)});
			}
			this.propagate('init');
		},
		gotoHash: function(_init){
			var hash = location.hash;
			if(!hash){return;}
			var selected;
			try {
				selected = this.atomElem.filter(hash);
			} catch(er){}
			if(!selected || !selected[0]){return;}
			selected = this.atomElem.index(selected[0]);
			if(_init){
				this.moveTo('goTo'+ selected, false, false, true);
			} else {
				this.moveTo('goTo'+ selected);
			}
			return true;
		},
		stageWidthUpdate: function(){
			var newDim1 = this.hidingWrapper[this.options.direction.dim]();
			
			if(newDim1 !== this.dims[1]){
				this.dims[1] = newDim1;
				this.maxPos = (this.dims[0] - this.dims[1]);
				this.updatePosition_Controls();
				if(this.hidingWrapper[this.options.direction.scroll]() > this.maxPos){
					this.moveTo(this.maxPos, false);
				}
				return true;
			}
			return false;
		},
		updateViewPagination: function(){
			var  o = this.options;
			var that = this;
			if(!this.pagination || o.paginationType != 'view' || !this.pagination[0]){return;}
			var content = '<ul>';
			var slides = Math.ceil(this.dims[0] / this.dims[1]);
			if(this.paginationSlides === slides){return;}
			this.paginationSlides = slides;
			for(var i = 0; i < slides; i++){
				content += '<li data-index="'+i+'"><button><span>'+ (i + 1) +'</span></button></li>'; 
			}
			this.pagination.html(content+'</ul>');
			$('> ul', this.pagination).delegate('li', 'click', function(){
				var index = this.getAttribute('data-index');
				if(index){
					that.moveTo.call(that, index * that.dims[1]);
				}
				return false;
			});
		},
		createPagination: function(){
			//paginationType
			var content = '<ul>', that = this, tmpContent, o = this.options;
			this.pagination = $(o.pagination, this.element[0]);
			if(this.pagination[0]){
				if(o.paginationType == 'view'){
					this.updateViewPagination();
					
				} else { // == 'teaser'
					this.atomElem.each(function(i){
						tmpContent = o.paginationAtoms.replace(/\$number/g, i + 1);
						content += (o.paginationTitleFrom) ? tmpContent.replace(/\$title/g, ($(o.paginationTitleFrom, this) || '').text().replace(/"/g, '&quot;')) : tmpContent;
					});
					this.pagination.html(content + '</ul>').find('li').each(function(i){
						$(this).click(function(){
							that.stopDiashow.call(that);
							that.moveTo.call(that, 'goTo' + i);
							return false;
						});
					});
				}
			}
		},
		getIndexNearPos: function(nPos){
			var len = this.dims.length;
			while (len--) {
				if (nPos == this.dims[len]) {
					return len;
				}
				else if(nPos > this.dims[len]) {
					return len+1;
				}
			}
			return false;
		},
		inView: function(atom){
			var dir 		= this.options.direction,
				stageDim 	= this.dims[1],
				atomDim 	= atom[dir.outerD](),
				curPos		= this.hidingWrapper['scroll' + dir.dir](),
				atomPos 	= atom[0]['offset'+ dir.dir]
			;
			if(curPos > atomPos || stageDim < atomDim + atomPos - curPos){
				return atomPos;
			}
			return false;
		},
		scrollIntoView: function(atom){
			var inView = this.inView(atom);
			if(inView !== false){
				this.moveTo(inView);
			}
		},

		_setOption: function(k, v){
			var o = this.options;
			switch(k) {
				case 'enableMwheel' :
					if( !v && o.enableMwheel){
						this.hidingWrapper.unmwheelIntent();
					}
					break;
					
				case 'addSubPixel':
					if(o.addSubPixel !== v) {
						this.dims[0] -= o.addSubPixel;
						o.addSubPixel = v;
						this.dims[0] += o.addSubPixel;
						this.update();
					}
					break;
			}
			$.Widget.prototype._setOption.apply(this, arguments);
		},
		startDiashow: function(){
			var that = this;
			clearInterval(this.diaTimer);
			this.isDiashowStopped = false;
			this.diaTimer = setInterval(function(){
				((that.position === that.maxPos && that.options.type !== 'carousel') ? that.moveTo(0, false) : that.moveTo('-' + that.options.moveStep));
			}, this.options.diashow);
			this.propagate('diashowstarted');
		},
		stopDiashow: function(){
			this.isDiashowStopped = true;
			this.pauseDiashow();
			this.propagate('diashowstopped');
		},

		pauseDiashow: function() {
			clearInterval(this.diaTimer);
			setTimeout(function(){
				clearInterval(this.diaTimer);
			}, 9);	
			this.isDiashowPaused = true;
			this.propagate('diashowpaused');
		},
		updateOnImgLoad: function(){
			var loadingSize = 0;
			var that = this;
			$('img', this.element).each(function(){
				if(!this.complete){
					loadingSize++;
					$(this).one('load', function(){
						loadingSize--;
						if(!loadingSize){
							that.update(true);
						}
					});
				}
			});
		},
		update: function(hard){
			var that = this, jElm, o = this.options;
			var subpixels = 0;
			if (hard) {
				this.dims = [0];
				this.atomElem = $(o.atoms, this.element);
			}
			
			this.dims[1] = this.hidingWrapper.css({
				overflow: 'hidden',
				position: 'relative'
			})[o.direction.dim]();
			
			if(o.stageTeaser) {
				this.atomElem.outerWidth(this.hidingWrapper.width());
			}
			
			var from = this.dims.length - 2;
			
			for(var i = from, len = this.atomElem.length; i < len; i++){
				jElm = $(this.atomElem[i]);
				that.dims.push(that.dims[0]);
				that.dims[0] += jElm[o.direction.outerD](true);
			}
			
			
			this.maxPos = (this.dims[0] - this.dims[1]);
			
			if(o.addSubPixelPerTeaser){
				subpixels = o.addSubPixelPerTeaser * this.atomElem.length;
			}
			var d = this.dims[0] + subpixels + o.addSubPixel;
			if(o.dynamicDim) {
				var newdim = d / $.testEm().emPx;
				if(newdim % 1 > 0) {
					d = parseInt(newdim, 10) +1;
				} else {
					d = newdim;
				}
				d += 'em';
			}
			this.moveElem.css(o.direction.dim, d);
			
			if (o.pagination) {
				this.createPagination(hard);
			}
			if(hard){
				this.moveTo('goTo'+ this.atomPos, false, false);
			}
			this.updatePosition_Controls();
			
		},
		updatePosition_Controls: function(pos){
			//calculate the curent position
			var o = this.options;
			var viewPos;
			pos = (isNaN(pos)) ? parseInt(this.hidingWrapper[o.direction.scroll](), 10) : pos;
			
			function changeState(elem, active){
				var doo = (active) ?{
						style: 'addClass'
					} : {
						style: 'removeClass'
					};
				return elem[doo.style](o.activeLinkClass);
			}
			
			if(pos !== this.position){
				this.percentage = pos / (this.maxPos / 100);
				this.oldPosition = this.position;
				this.oldAtomPos = this.atomPos;
				this.position = pos;
				
				var num = this.getIndexNearPos(this.position);
			   
				num = (num) ? num - 2 : 0;
				this.atomPos = num;
			}
			
			viewPos = this.atomPos;
			
			this.percentage = pos / (this.maxPos / 100);
			
			if (pos <= this.minPos && this.prevLink.hasClass(o.activeLinkClass)) {
				o.linkFn.call(this.prevLink, 'hide', this.ui());
				changeState(this.prevLink);
				this.propagate('startreached');
			} else if (pos > this.minPos && !this.prevLink.hasClass(o.activeLinkClass)) {
				o.linkFn.call(this.prevLink, 'show', this.ui());
				changeState(this.prevLink, true);
				this.propagate('startleft');
			}
			if (pos >= this.maxPos && this.nextLink.hasClass(o.activeLinkClass)) {
				o.linkFn.call(this.nextLink, 'hide', this.ui());
				changeState(this.nextLink);
				this.propagate('endreached');
				
			} else if (pos < this.maxPos && !this.nextLink.hasClass(o.activeLinkClass)) {
				o.linkFn.call(this.nextLink, 'show', this.ui());
				changeState(this.nextLink, true);
				this.propagate('endleft');
			}
			if (this.pagination) {
				var oldActive = this.pagination.find('li')
						.filter('.' + o.activePaginationClass)
						.removeClass(o.activePaginationClass), 
					newActive
				;
				if(o.paginationType == 'view'){
					if(this.maxPos == pos) {
						viewPos = this.paginationSlides - 1;
					} else {
						viewPos = Math.round(pos / this.dims[1]);
					}
				}
				
				newActive = oldActive.end().eq(viewPos).addClass(o.activePaginationClass);
				if ($.isFunction(o.paginationFn)) {
					o.paginationFn.call(oldActive, 'inactive');
					o.paginationFn.call(newActive, 'active');
				}
			}
		},
		getNummericPosition: function(ePos){
			var rel = false, num, lastDim = this.dims[this.dims.length - 1];
			
			// handle Atom Step & goTo
			if (ePos.indexOf('goTo') === 0) {
				num = parseInt(/(\d+)$/.exec(ePos)[0], 10) + 2;
				ePos = this.dims[num];
			}
			else if (ePos.indexOf('centerTo') === 0) {
				num = parseInt(/(\d+)$/.exec(ePos)[0], 10) + 2;
				ePos = this.dims[num] - (this.dims[1] / 2) + (this.atomElem.filter(":eq("+num+")")[this.options.direction.outerD]() / 2);
			} 
			else if (ePos.indexOf('stageWidth') === 1) {
				var that = this;
				rel = ePos.slice(0, 1);
				num = this.dims[1];
				var amount = (function() {
					var i = 2;
					for(i; i < that.dims.length; i++) {
						if(that.dims[i] > that.dims[1]) {
							break;
						}
					}

					return i-3; // minus 1 for the teaser that we are actually too far, and minus 2 for the first entries of dims.. makes 3
				})();
				if(this.atomPos === 0 && rel === '+') { return; }
				ePos = rel === '+' ?  (this.atomPos - amount + 2) < 2 ? 2 : (this.atomPos - amount+2) : (this.atomPos + amount + 2);
				ePos = this.dims[ePos];
			}
			else 
				if (ePos == '-atom' || ePos == '-atom1') {
					num = this.atomPos + 3;
					ePos = (this.dims[num] || this.dims[num] === 0) ? this.dims[num] : lastDim;
				}
				else 
					if (ePos == '+atom' || ePos == '+atom1') {
						ePos = (this.atomPos) ? this.dims[this.atomPos + 1] : 0;
					}
					else 
						if (ePos.indexOf('atom') == 1) {
							num = parseInt(/(\d+)$/.exec(ePos)[0], 10);
							if (ePos.indexOf('-') === 0) {
								num += 2;
								if (this.dims[this.atomPos + num]) {
									ePos = this.dims[this.atomPos + num];
								}
								else {
									ePos = lastDim;
								}
							}
							else {
								num -= 2;
								var aLen = this.atomPos - num;
								if (aLen > 1 && this.dims[this.atomPos - num]) {
									ePos = this.dims[this.atomPos - num];
								}
								else {
									ePos = 0;
								}
							}
						// handle: +/-Number
						}
						else 
							if (ePos.indexOf('+') === 0 || ePos.indexOf('-') === 0) {
								rel = ePos.slice(0, 1);
								ePos = parseInt(ePos.slice(1), 10);
								ePos = (rel == '-') ? this.position + ePos : this.position - ePos;
							}
							else {
								// handle Percentage
								var per = /(\d+)%$/.exec(ePos);
								if (per && per[1]) {
									ePos = this.maxPos / 100 * parseFloat(ePos);
								}
							}
			if(this.options.stickyFirstLast){
				if((ePos - this.maxPos) * -1 < this.atomElem.filter(':last')[this.options.direction.outerD]() && ePos > this.position){
					ePos = this.maxPos;
				} else if(ePos < this.atomElem[this.options.direction.outerD]() && ePos < this.position){
					ePos = 0;
				}
			}
			return ePos;
		},
		readjustPosition: function(hard){
			var scroll = this.options.direction.scroll;
			this.hidingWrapper[scroll](this.position);
			if(hard){
				this.hidingWrapper[scroll == 'scrollTop' ? 'scrollLeft' : 'scrollTop'](0);
			}
		},
		moveTo: function(pos, anim, animOp, _initForce){
			pos = (typeof pos === 'string' || isNaN(pos)) ? this.getNummericPosition(pos) : pos;
			pos = (pos <= 0) ? 0 : (pos >= this.maxPos) ? this.maxPos : pos;
			if (pos === this.position) {
				return false;
			}
			var o = this.options, scroll = o.direction.scroll;
			this.updatePosition_Controls(pos);
			this.propagate('start', this.oldPosition);
			
			anim = (typeof anim == 'undefined') ? o.animate : anim;
			if (anim) {
				//dirty break recursion
				animOp = animOp ||
				{};
				animOp = $.extend({}, o.animateOptions, {
					slide: this
				}, animOp);
				var animCss = (scroll == 'scrollTop') ? 
					{scrollerTop: pos} : 
					{scrollerLeft: pos}
				;
				this.hidingWrapper.stop().animate(animCss, animOp);
			}
			else {
				this.hidingWrapper.stop()[scroll](pos);
				if(_initForce){
					var that = this;
					setTimeout(function(){
						if(that.hidingWrapper[scroll]() != pos){
							that.hidingWrapper[scroll](pos);
						}
					}, 9);
				}
				this.propagate('end');
			}
		},
		ui: function(){
			return {
				instance: this,
				options: this.options,
				pos: this.position,
				percentPos: this.percentage,
				oldIndex: this.oldAtomPos,
				newIndex: this.atomPos,
				size: this.dims.length - 2
			};
		},
		propagate: function(n, pos){
			var fx = {};
			if(n == 'slide'){
				fx = pos;
				pos = fx.now;
			}
			var args = (pos || pos === 0) ? $.extend(this.ui(), {
				'pos': pos,
				'fx': fx,
				percentPos: pos / (this.maxPos / 100)
			}) : this.ui();
			if(n === 'start'){
				this.isSliding = true;
			} else if(n === 'end'){
				this.isSliding = false;
			}
			if(n == 'slide'){
				this.element.triggerHandler("scroller" + n, [args]);
			} else {
				this.element.trigger("scroller" + n, [args]);
			}
			if(this.options[n]){
				this.options[n].call(this.element[0], {type: 'scroller' + n}, args);
			}
		}
	});
	$.each({scrollerLeft: 'scrollLeft', scrollerTop: 'scrollTop'}, function(name, prop){
		$.fx.step[name] = function(fx){
			if (fx.now || fx.now === 0) {
				var scroller = fx.options.slide;
				if (scroller) {
					if(!fx.scrollerInit){
						fx.scrollerInit = true;
						fx.start = scroller.hidingWrapper[prop]();
						fx.now = fx.start;
					}
					scroller.hidingWrapper[prop](fx.now);
				   	scroller.propagate('slide', fx);
				}
			}
		};
	});
})(jQuery);
